/****************************************************************/
/*                                                              */
/*  file: altrobus_scope.c                                     */
/*                                                              */
/*  Author: Luciano Musa, CERN-EP                               */
/*                                                              */
/*  20. Jul. 05  last  update                                   */
/*                                                              */
/****************************************************************/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/file.h>
#include <signal.h>
#include <tools/get_input.h>
#include <tools/rcc_time_stamp.h>
#include "u2f.h"
#include "fec.h"

/*************/
/* Externals */
/*************/
extern u_int traceLevel;


/*************/
/* Prototypes*/
/*************/
int mainhelp(void);
int altrobus_menu(void);



/****************/
int mainhelp(void)
/****************/
{
  printf("\nCall Luciano Musa, 7676261, 163119 if you need help\n");
  return(0);
}

/****************/
int debug_mode(void)
/****************/
{
  printf("Setting the debugLevel:\n");
  printf("0: No debug\n");
  printf("1: messages from altrobus_scope\n");
  printf("2: messages altrobus_scope and fec_lib\n");
  printf("======================================\n");
  printf("Your choice ");
  traceLevel = getdecd(traceLevel);
  return(0);
}

/******************************/
int main(int argc, char *argv[])
/******************************/
{  
  static int ret, fun = 1;
  traceLevel =0;
  
  ret = ts_open(1, TS_DUMMY);
  if (ret)
    rcc_error_print(stdout, ret);
 
  printf("\n\nThis is ALTRO BUS SCOPE\n");
  while(fun != 0)
  {
    printf("\n");
    printf("Select an option:\n");
    printf("  1 Print help\n");
    printf("  2 ALTROBUS menu\n");
    printf("  3 Set debugging parameters\n");
    printf("  0 Quit\n");
    printf("Your choice ");
    fun = getdecd(fun);
    if (fun == 1) mainhelp();
    if (fun == 2) altrobus_menu();
    if (fun == 3) debug_mode();
    
  }

  ts_close(TS_DUMMY);
  exit(0);
}


/*******************/
int altrobus_menu(void)
/*******************/
{  
  static u_int ret, fun;

  static u_int number_fec = 0, act_fec =0;
  
  static u_int channel_address =0, fec_address =0, fec_channel_address=0;
  static u_int reg_address = 0, reg_data =0, read_data =0, num_cycles=1, num_samples =128;
  
  static u_int debug =0;
  static u_int rcode=0, rdata=0, ccode =0, cdata =0;

  u_int fec_status, hadd, error =0;
  u_int loop, tick;
  
  static char fname[200] = "/home/rcu/NEW_U2F/altro_usb/log_files/backplane_test.log";
  char data_fname[200] = "/home/rcu/NEW_U2F/altro_usb/data_files/test_data";
  char finame[200] = "/home/rcu/NEW_U2F/altro_usb/pedestal/ped_in";
  FILE *file;


  /* Open U2F_LIB */
  ret = U2F_Open(HW, DEFAULT_NODE);
  if (ret)
    rcc_error_print(stdout, ret);

  if(traceLevel > 0)
    debug =1;
 
  fun = 1; 
  while(fun != 0)
  {
    printf("\n");
    printf("Select an option:\n");
    printf("   1 CONFIGURE THE SYSTEM   	2 RESET FECs\n");
    printf("   3 READ_ALTRO_REG         	4 WRITE_ALTRO_REG\n");
    printf("   5 READ_BC_REG            	6 WRITE_BC_REG\n");
    printf("   7 READ_FEC_ADD\n");
    printf("   8 WRITE_READ_CHECK_ALTRO_REG\n");
    printf("   9 WRITE_READ_CHECK_BC\n");
    printf("  10 FEC_READOUT\n");
    printf("  =========================================================\n");
    printf("   0 Quit\n");
    printf("Your choice ");
    fun = getdecd(fun);
    
    if((fun < 0) || (fun > 10))
    {
      printf ("\n ERROR: your selection is out of range\n");  
      return(-1);
    }

    if (fun ==1)
    {
      printf("\nNr of FECs ON ");
      number_fec = getdecd(number_fec);

      act_fec = 0;
      for(loop = 0; loop < number_fec; loop++)
      {
        printf("Enter Addres of FEC nr. %d ", loop);
        fec_address = getdecd(fec_address); 
	act_fec = (act_fec) | (1 << fec_address);  
      }
      /* Switch on FEC with Address = 0  */
      rcode = O_ACTFEC;
      rdata = act_fec;
      ret = U2F_Reg_Write(rcode, rdata);
      if (ret)
        rcc_error_print(stdout, ret);

      /* Reading back the ACTFEC Register */
      rcode = O_ACTFEC;
      ret = U2F_Reg_Read(rcode, &rdata);
      if (ret)
        rcc_error_print(stdout, ret);
  
      printf("\nThe FECs have the following power status:\n");
      for(loop = 0; loop < 13; loop++)
      {
        fec_status = (rdata >> loop) & 0x1;
	if(fec_status)
	  printf("FEC(0x%x)=ON\n", loop);
	else
	  printf("FEC(0x%x)=OFF\n", loop);
      }
      	  
      /* delay 100ms */
      ts_delay(100000);

      /* Reset FECs */
      ccode = C_FECRST;
      cdata = 0;
      ret = U2F_Exec_Command(ccode, cdata);
      if (ret)
        rcc_error_print(stdout, ret);

      /* delay 1ms */
      ts_delay(1000);
    }
    
    if (fun ==2)
    {
      /* delay 1s */
      ts_delay(1000000);

      /* Reset FECs twice*/
      ccode = C_FECRST;
      cdata = 0;
      ret = U2F_Exec_Command(ccode, cdata);
      if (ret)
        rcc_error_print(stdout, ret);

      ret = U2F_Exec_Command(ccode, cdata);
      if (ret)
        rcc_error_print(stdout, ret);
  
      /* delay 1ms */
      ts_delay(1000);
    }


    if (fun == 3) 
    {
      printf("Enter the FEC Address");
      fec_address = getdecd(fec_address);

      printf("Enter the Channel Address");
      channel_address = getdecd(channel_address);
      
      printf("Enter the Register Address");
      reg_address = gethexd(reg_address);

      printf("Enter the Number of Read Cycles (0 -> infinite loop) ");
      num_cycles = getdecd(num_cycles);

      channel_address = ((fec_address << 7) & 0xf80) | (fec_channel_address & 0x7f);
      if(traceLevel == 2)
        printf("Channel Address = %x\n", channel_address);

      if(num_cycles)
      { 
        for(loop = 0; loop < num_cycles; loop++)
        {      
          ret = Read_ALTRO_Reg(debug, channel_address, reg_address, &reg_data);
          if(ret)
            rcc_error_print(stdout, ret);
	  printf("\nReg %x, of FEC Channel %d, of FEC %d, has value: 0x%x\n", reg_address, fec_channel_address, fec_address, reg_data);   
        }
      }
    
      else
      {
        tick = 0;
        while(!num_cycles)
	{
           tick++;
           ret = Read_ALTRO_Reg(debug, channel_address, reg_address, &reg_data);
           if(ret)
             rcc_error_print(stdout, ret);
	   if(tick == 10000)
	   {
	     printf("\nLoop of 10000 Cycles Completed");
             printf("\nReg %x, of FEC Channel %d, of FEC %d, has value: 0x%x\n", reg_address, fec_channel_address, fec_address, reg_data);   
	     tick =0;
	   }
	 }
       }  
    }	



    if (fun == 4) 
    {
      printf("Enter the FEC Address");
      fec_address = getdecd(fec_address);

      printf("Enter the Channel Address");
      channel_address = getdecd(channel_address);
      
      printf("Enter the Register Address");
      reg_address = gethexd(reg_address);

      printf("Enter the Register Data");
      reg_data = gethexd(reg_data);
       	
      printf("Enter the Number of Write Cycles (0 -> infinite loop) ");
      num_cycles = getdecd(num_cycles);

      channel_address = ((fec_address << 7) & 0xf80) | (fec_channel_address & 0x7f);
      if(traceLevel == 2)
        printf("Channel Address = %x\n", channel_address);

      if(num_cycles)
      { 
        for(loop = 0; loop < num_cycles; loop++)
        {      
          ret = Write_ALTRO_Reg(debug, channel_address, reg_address, reg_data);
          if(ret)
            rcc_error_print(stdout, ret);
        }
      }
    
      else
      {
        tick = 0;
        while(!num_cycles)
	{
           tick++;
           ret = Write_ALTRO_Reg(debug, channel_address, reg_address, reg_data);
           if(ret)
             rcc_error_print(stdout, ret);
	   if(tick == 10000)
	   {
	     printf("\nLoop of 10000 Cycles Completed\n");
	     tick =0;
	   }
	 }
       }  
    }	


    if (fun == 5) 
    {
      printf("Enter the FEC Address");
      fec_address = getdecd(fec_address);
    
      printf("Enter the BC Register Address");
      reg_address = gethexd(reg_address);

      printf("Enter the Number of Read Cycles (0 -> infinite loop) ");
      num_cycles = getdecd(num_cycles);

      if(num_cycles)
      { 
        for(loop = 0; loop < num_cycles; loop++)
        {      

          ret = FEC_BC_Reg_Read(debug, fec_address, reg_address, &reg_data);
          if(ret)
            rcc_error_print(stdout, ret);
	  printf("\nBC Reg %x, of FEC %d, has value: 0x%x\n", reg_address, fec_address, reg_data);   
        }
      }
    
      else
      {
        tick = 0;
        while(!num_cycles)
	{
           tick++;
           ret = FEC_BC_Reg_Read(debug, fec_address, reg_address, &reg_data);
           if(ret)
             rcc_error_print(stdout, ret);
	   if(tick == 10000)
	   {
	     printf("\nLoop of 10000 Cycles Completed");
             printf("\nBC Reg %x, of FEC %d, has value: 0x%x\n", reg_address, fec_address, reg_data);   
	     tick =0;
	   }
	 }
       }  
    }	



    if (fun == 6) 
    {
      printf("Enter the FEC Address");
      fec_address = getdecd(fec_address);
      
      printf("Enter the BC Register Address");
      reg_address = gethexd(reg_address);

      printf("Enter the Register Data");
      reg_data = gethexd(reg_data);
       	
      printf("Enter the Number of Write Cycles (0 -> infinite loop) ");
      num_cycles = getdecd(num_cycles);

      if(num_cycles)
      { 
        for(loop = 0; loop < num_cycles; loop++)
        {      
          ret = FEC_BC_Reg_Write(debug, fec_address, reg_address, reg_data);
          if(ret)
            rcc_error_print(stdout, ret);
        }
      }
    
      else
      {
        tick = 0;
        while(!num_cycles)
	{
           tick++;
           ret = FEC_BC_Reg_Write(debug, fec_address, reg_address, reg_data);
           if(ret)
             rcc_error_print(stdout, ret);
	   if(tick == 10000)
	   {
	     printf("\nLoop of 10000 Cycles Completed\n");
	     tick =0;
	   }
	 }
       }  
    }	



    if (fun == 7) 
    {
      printf("Enter the FEC Address");
      fec_address = getdecd(fec_address);
    
      printf("Enter the Number of Read Cycles (0 -> infinite loop) ");
      num_cycles = getdecd(num_cycles);

      if(num_cycles)
      { 
        for(loop = 0; loop < num_cycles; loop++)
        {      

          ret = FEC_BC_Reg_Read(debug, fec_address, 0x13, &reg_data);
          if(ret)
            rcc_error_print(stdout, ret);
	  hadd = (reg_data & 0xf800) >> 11;
	  printf("\nAt Address 0x%x respends FEC with HADD 0x%x\n", fec_address, hadd);   
        }
      }
    
      else
      {
        tick = 0;
        while(!num_cycles)
	{
           tick++;
           ret = FEC_BC_Reg_Read(debug, fec_address, 0x13, &reg_data);
           if(ret)
             rcc_error_print(stdout, ret);
	   if(tick == 10000)
	   {
	     printf("\nLoop of 10000 Cycles Completed");
             hadd = reg_data;
	     printf("\nAt Address 0x%x respends FEC with HADD 0x%x\n", fec_address, hadd);   
	     tick =0;
	   }
	 }
       }  
    }	


    if (fun == 8) 
    {
      printf("Enter the FEC Address");
      fec_address = getdecd(fec_address);

      printf("Enter the Channel Address");
      channel_address = getdecd(channel_address);
      
      printf("Enter the Register Address");
      reg_address = gethexd(reg_address);

      printf("Enter the Register Data");
      reg_data = gethexd(reg_data);
       	
      printf("Enter the Number of Write/Read Cycles (0 -> infinite loop) ");
      num_cycles = getdecd(num_cycles);

      channel_address = ((fec_address << 7) & 0xf80) | (fec_channel_address & 0x7f);
      if(traceLevel == 2)
        printf("Channel Address = %x\n", channel_address);

      if(num_cycles)
      { 
        for(loop = 0; loop < num_cycles; loop++)
        {      
          ret = Write_ALTRO_Reg(debug, channel_address, reg_address, reg_data);
          if(ret)
            rcc_error_print(stdout, ret);
	    
          ret = Read_ALTRO_Reg(debug, channel_address, reg_address, &read_data);
          if(ret)
            rcc_error_print(stdout, ret);

          if(read_data != reg_data)
          {
	    printf("Test error\n");
            file = fopen(fname,"a");
            fprintf(file, "Error in cycle Nr. %d\n", loop);
            fprintf(file, "Written: 0x%x;\t\tRead: %x\n", reg_data, read_data);
	    fclose(file);
          }	
        }
      }
    
      else
      {
        tick = 0;
        while(!num_cycles)
	{
           tick++;
	   
           ret = Write_ALTRO_Reg(debug, channel_address, reg_address, reg_data);
           if(ret)
             rcc_error_print(stdout, ret);

           ret = Read_ALTRO_Reg(debug, channel_address, reg_address, &read_data);
           if(ret)
             rcc_error_print(stdout, ret);

           if(read_data != reg_data)
           {
      	     printf("Test error\n");
             error++;
             file = fopen(fname,"a");
             fprintf(file, "Error in cycle Nr. %d\n", tick++);
             fprintf(file, "Written: 0x%x;\t\tRead: %x\n", reg_data, read_data);
           }	

	   if(tick == 10000)
	   {
	     printf("\nLoop of 10000 Cycles Completed\n");
	     tick =0;
	   }
	   
	   if(error == 1000)
	   {
	     printf("\n\n THE SYSTEM IS MALFUNCTIONING\n\n");
	     exit(0);
	   } 
	 }
       }  
    }  /* end of function */ 	


    if (fun == 10) 
    {
      printf("Enter the FEC Address");
      fec_address = getdecd(fec_address);

      printf("Enter the Number of Events");
      num_cycles = getdecd(num_cycles);
      
      printf("Enter the Number of Samples");
      num_samples = getdecd(num_samples); 

      ret = Write_ALTRO_PMEM(1, 1, 0, 128, finame);
      if(ret)
        rcc_error_print(stdout, ret);

      printf("\nCalling FEC_Event_Readout\n");
      ret = FEC_Event_Readout_2(debug, fec_address, 1, 0, num_cycles, num_samples, data_fname); 

    }  /* end of function */ 	 


  } /* end of while */

  ret = U2F_Close();
  return(0);
}
